package fr.inria.phoenix.diasuite.framework.context.alert;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.device.doorlocker.StateFromDoorLocker;

/**
 * Detect urgent events such as a forced door, a stuck door, or a blackout
 * 
 * <pre>
 * context Alert as String {
 * 	when provided state from DoorLocker maybe publish;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractAlert extends Service {
    
    public AbstractAlert(ServiceConfiguration serviceConfiguration) {
        super("/Context/Alert/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        postInitialize();
    }
    
    @Override
    protected void postInitialize() {
        // Default implementation of post initialize: subscribe to all required devices
        discoverDoorLockerForSubscribe.all().subscribeState(); // subscribe to state from all DoorLocker devices
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("state") && source.isCompatible("/Device/Device/PhysicalDevice/DoorLocker/")) {
            StateFromDoorLocker stateFromDoorLocker = new StateFromDoorLocker(this, source, (fr.inria.phoenix.diasuite.framework.datatype.state.State) value);
            
            AlertValuePublishable returnValue = onStateFromDoorLocker(stateFromDoorLocker);
            if(returnValue != null && returnValue.doPublish()) {
                setAlert(returnValue.getValue());
            }
        }
    }
    
    @Override
    public final Object getValueCalled(java.util.Map<String, Object> properties, RemoteServiceInfo source, String valueName,
            Object... indexes) throws Exception {
        if (valueName.equals("alert")) {
            return getLastValue();
        }
        throw new InvocationException("Unsupported method call: " + valueName);
    }
    // End of methods from the Service class
    
    // Code relative to the return value of the context
    private java.lang.String contextValue;
    
    private void setAlert(java.lang.String newContextValue) {
        contextValue = newContextValue;
        getProcessor().publishValue(getOutProperties(), "alert", newContextValue);
    }
    
    /**
     * Get the last value of the context
     * 
     * @return the latest value published by the context
     */
    protected final java.lang.String getLastValue() {
        return contextValue;
    }
    
    /**
     * A class that represents a value that might be published for the <code>Alert</code> context. It is used by
     * event methods that might or might not publish values for this context.
     */
    protected final static class AlertValuePublishable {
        
        // The value of the context
        private java.lang.String value;
        // Whether the value should be published or not
        private boolean doPublish;
        
        public AlertValuePublishable(java.lang.String value, boolean doPublish) {
            this.value = value;
            this.doPublish = doPublish;
        }
        
        /**
         * @return the value of the context that might be published
         */
        public java.lang.String getValue() {
            return value;
        }
        
        /**
         * Sets the value that might be published
         * 
         * @param value the value that will be published if {@link #doPublish()} returns true
         */
        public void setValue(java.lang.String value) {
            this.value = value;
        }
        
        /**
         * @return true if the value should be published
         */
        public boolean doPublish() {
            return doPublish;
        }
        
        /**
         * Set the value to be publishable or not
         * 
         * @param doPublish if true, the value will be published
         */
        public void setDoPublish(boolean doPublish) {
            this.doPublish = doPublish;
        }
    }
    // End of code relative to the return value of the context
    
    // Interaction contract implementation
    /**
     * This method is called when a <code>DoorLocker</code> device on which we have subscribed publish on its <code>state</code> source.
     * 
     * <pre>
     * when provided state from DoorLocker maybe publish;
     * </pre>
     * 
     * @param stateFromDoorLocker the value of the <code>state</code> source and the <code>DoorLocker</code> device that published the value.
     * @return a {@link AlertValuePublishable} that says if the context should publish a value and which value it should publish
     */
    protected abstract AlertValuePublishable onStateFromDoorLocker(StateFromDoorLocker stateFromDoorLocker);
    
    // End of interaction contract implementation
    
    // Discover part for DoorLocker devices
    /**
     * Use this object to discover DoorLocker devices.
     * <p>
     * batteryLevel
     * 
     * <pre>
     * device DoorLocker extends PhysicalDevice {
     * 	action Lock;
     * 	action UnLock;
     * 	source locked as Boolean;
     * 	source state as State; 
     * 	source batstate as State; 
     * }
     * </pre>
     * 
     * @see DoorLockerDiscoverer
     */
    protected final DoorLockerDiscoverer discoverDoorLockerForSubscribe = new DoorLockerDiscoverer(this);
    
    /**
     * Discover object that will exposes the <code>DoorLocker</code> devices that can be discovered
     * <p>
     * batteryLevel
     * 
     * <pre>
     * device DoorLocker extends PhysicalDevice {
     * 	action Lock;
     * 	action UnLock;
     * 	source locked as Boolean;
     * 	source state as State; 
     * 	source batstate as State; 
     * }
     * </pre>
     */
    protected final static class DoorLockerDiscoverer {
        private Service serviceParent;
        
        private DoorLockerDiscoverer(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private DoorLockerComposite instantiateComposite() {
            return new DoorLockerComposite(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>DoorLocker</code> devices
         * 
         * @return a {@link DoorLockerComposite} object composed of all discoverable <code>DoorLocker</code>
         */
        public DoorLockerComposite all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>DoorLocker</code> devices
         * 
         * @return a {@link DoorLockerProxy} object pointing to a random discoverable <code>DoorLocker</code> device
         */
        public DoorLockerProxy anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>DoorLocker</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link DoorLockerComposite} object composed of all matching <code>DoorLocker</code> devices
         */
        public DoorLockerComposite whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>DoorLocker</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return a {@link DoorLockerComposite} object composed of all matching <code>DoorLocker</code> devices
         */
        public DoorLockerComposite whereLocation(java.lang.String location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
        
        /**
         * Returns a composite of all accessible <code>DoorLocker</code> devices whose attribute <code>user</code> matches a given value.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return a {@link DoorLockerComposite} object composed of all matching <code>DoorLocker</code> devices
         */
        public DoorLockerComposite whereUser(java.lang.String user) throws CompositeException {
            return instantiateComposite().andUser(user);
        }
    }
    
    /**
     * A composite of several <code>DoorLocker</code> devices
     * <p>
     * batteryLevel
     * 
     * <pre>
     * device DoorLocker extends PhysicalDevice {
     * 	action Lock;
     * 	action UnLock;
     * 	source locked as Boolean;
     * 	source state as State; 
     * 	source batstate as State; 
     * }
     * </pre>
     */
    protected final static class DoorLockerComposite extends fr.inria.diagen.core.service.composite.Composite<DoorLockerProxy> {
        private DoorLockerComposite(Service serviceParent) {
            super(serviceParent, "/Device/Device/PhysicalDevice/DoorLocker/");
        }
        
        @Override
        protected DoorLockerProxy instantiateProxy(RemoteServiceInfo rsi) {
            return new DoorLockerProxy(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link DoorLockerComposite}, filtered using the attribute <code>id</code>.
         */
        public DoorLockerComposite andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return this {@link DoorLockerComposite}, filtered using the attribute <code>location</code>.
         */
        public DoorLockerComposite andLocation(java.lang.String location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>user</code>.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return this {@link DoorLockerComposite}, filtered using the attribute <code>user</code>.
         */
        public DoorLockerComposite andUser(java.lang.String user) throws CompositeException {
            filterByAttribute("user", user);
            return this;
        }
        
        /**
         * Subscribes to the <code>state</code> source. After a call to this method, the context will be notified when a
         * <code>DoorLocker</code> device of this composite publishes a value on its <code>state</code> source.
         */
        public void subscribeState() {
            subscribeValue("state");
        }
        
        /**
         * Unsubscribes from the <code>state</code> source. After a call to this method, the context will no more be notified
         * when a <code>DoorLocker</code> device of this composite publishes a value on its <code>state</code> source.
         */
        public void unsubscribeState() {
            unsubscribeValue("state");
        }
    }
    
    /**
     * A proxy to one <code>DoorLocker</code> device
     * <p>
     * batteryLevel
     * 
     * <pre>
     * device DoorLocker extends PhysicalDevice {
     * 	action Lock;
     * 	action UnLock;
     * 	source locked as Boolean;
     * 	source state as State; 
     * 	source batstate as State; 
     * }
     * </pre>
     */
    protected final static class DoorLockerProxy extends Proxy {
        private DoorLockerProxy(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Subscribes to the <code>state</code> source. After a call to this method, the context will be notified when the
         * <code>DoorLocker</code> device of this proxy publishes a value on its <code>state</code> source.
         */
        public void subscribeState() {
            subscribeValue("state");
        }
        
        /**
         * Unsubscribes from the <code>state</code> source. After a call to this method, the context will no more be notified
         * when the <code>DoorLocker</code> device of this proxy publishes a value on its <code>state</code> source.
         */
        public void unsubscribeState() {
            unsubscribeValue("state");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
    // End of discover part for DoorLocker devices
}
