package fr.inria.phoenix.diasuite.framework.context.format;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.device.doorlocker.BatstateFromDoorLocker;
import fr.inria.phoenix.diasuite.framework.device.doorlocker.LockedFromDoorLocker;
import fr.inria.phoenix.diasuite.framework.device.doorlocker.StateFromDoorLocker;

/**
 * * DoorLocks contexts
 * Format various events for logging purpose
 * 
 * <pre>
 * context Format as Event {
 * 	when provided locked from DoorLocker always publish;
 * 	when provided batstate from DoorLocker always publish;
 * 	when provided state from DoorLocker always publish;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractFormat extends Service {
    
    public AbstractFormat(ServiceConfiguration serviceConfiguration) {
        super("/Context/Format/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        postInitialize();
    }
    
    @Override
    protected void postInitialize() {
        // Default implementation of post initialize: subscribe to all required devices
        discoverDoorLockerForSubscribe.all().subscribeBatstate(); // subscribe to batstate from all DoorLocker devices
        discoverDoorLockerForSubscribe.all().subscribeLocked(); // subscribe to locked from all DoorLocker devices
        discoverDoorLockerForSubscribe.all().subscribeState(); // subscribe to state from all DoorLocker devices
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("batstate") && source.isCompatible("/Device/Device/PhysicalDevice/DoorLocker/")) {
            BatstateFromDoorLocker batstateFromDoorLocker = new BatstateFromDoorLocker(this, source, (fr.inria.phoenix.diasuite.framework.datatype.state.State) value);
            
            setFormat(onBatstateFromDoorLocker(batstateFromDoorLocker));
        }
        if (eventName.equals("locked") && source.isCompatible("/Device/Device/PhysicalDevice/DoorLocker/")) {
            LockedFromDoorLocker lockedFromDoorLocker = new LockedFromDoorLocker(this, source, (java.lang.Boolean) value);
            
            setFormat(onLockedFromDoorLocker(lockedFromDoorLocker));
        }
        if (eventName.equals("state") && source.isCompatible("/Device/Device/PhysicalDevice/DoorLocker/")) {
            StateFromDoorLocker stateFromDoorLocker = new StateFromDoorLocker(this, source, (fr.inria.phoenix.diasuite.framework.datatype.state.State) value);
            
            setFormat(onStateFromDoorLocker(stateFromDoorLocker));
        }
    }
    
    @Override
    public final Object getValueCalled(java.util.Map<String, Object> properties, RemoteServiceInfo source, String valueName,
            Object... indexes) throws Exception {
        if (valueName.equals("format")) {
            return getLastValue();
        }
        throw new InvocationException("Unsupported method call: " + valueName);
    }
    // End of methods from the Service class
    
    // Code relative to the return value of the context
    private fr.inria.phoenix.diasuite.framework.datatype.event.Event contextValue;
    
    private void setFormat(fr.inria.phoenix.diasuite.framework.datatype.event.Event newContextValue) {
        contextValue = newContextValue;
        getProcessor().publishValue(getOutProperties(), "format", newContextValue);
    }
    
    /**
     * Get the last value of the context
     * 
     * @return the latest value published by the context
     */
    protected final fr.inria.phoenix.diasuite.framework.datatype.event.Event getLastValue() {
        return contextValue;
    }
    // End of code relative to the return value of the context
    
    // Interaction contract implementation
    /**
     * This method is called when a <code>DoorLocker</code> device on which we have subscribed publish on its <code>locked</code> source.
     * 
     * <pre>
     * when provided locked from DoorLocker always publish;
     * </pre>
     * 
     * @param lockedFromDoorLocker the value of the <code>locked</code> source and the <code>DoorLocker</code> device that published the value.
     * @return the value to publish
     */
    protected abstract fr.inria.phoenix.diasuite.framework.datatype.event.Event onLockedFromDoorLocker(LockedFromDoorLocker lockedFromDoorLocker);
    
    /**
     * This method is called when a <code>DoorLocker</code> device on which we have subscribed publish on its <code>batstate</code> source.
     * 
     * <pre>
     * when provided batstate from DoorLocker always publish;
     * </pre>
     * 
     * @param batstateFromDoorLocker the value of the <code>batstate</code> source and the <code>DoorLocker</code> device that published the value.
     * @return the value to publish
     */
    protected abstract fr.inria.phoenix.diasuite.framework.datatype.event.Event onBatstateFromDoorLocker(BatstateFromDoorLocker batstateFromDoorLocker);
    
    /**
     * This method is called when a <code>DoorLocker</code> device on which we have subscribed publish on its <code>state</code> source.
     * 
     * <pre>
     * when provided state from DoorLocker always publish;
     * </pre>
     * 
     * @param stateFromDoorLocker the value of the <code>state</code> source and the <code>DoorLocker</code> device that published the value.
     * @return the value to publish
     */
    protected abstract fr.inria.phoenix.diasuite.framework.datatype.event.Event onStateFromDoorLocker(StateFromDoorLocker stateFromDoorLocker);
    
    // End of interaction contract implementation
    
    // Discover part for DoorLocker devices
    /**
     * Use this object to discover DoorLocker devices.
     * <p>
     * batteryLevel
     * 
     * <pre>
     * device DoorLocker extends PhysicalDevice {
     * 	action Lock;
     * 	action UnLock;
     * 	source locked as Boolean;
     * 	source state as State; 
     * 	source batstate as State; 
     * }
     * </pre>
     * 
     * @see DoorLockerDiscoverer
     */
    protected final DoorLockerDiscoverer discoverDoorLockerForSubscribe = new DoorLockerDiscoverer(this);
    
    /**
     * Discover object that will exposes the <code>DoorLocker</code> devices that can be discovered
     * <p>
     * batteryLevel
     * 
     * <pre>
     * device DoorLocker extends PhysicalDevice {
     * 	action Lock;
     * 	action UnLock;
     * 	source locked as Boolean;
     * 	source state as State; 
     * 	source batstate as State; 
     * }
     * </pre>
     */
    protected final static class DoorLockerDiscoverer {
        private Service serviceParent;
        
        private DoorLockerDiscoverer(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private DoorLockerComposite instantiateComposite() {
            return new DoorLockerComposite(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>DoorLocker</code> devices
         * 
         * @return a {@link DoorLockerComposite} object composed of all discoverable <code>DoorLocker</code>
         */
        public DoorLockerComposite all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>DoorLocker</code> devices
         * 
         * @return a {@link DoorLockerProxy} object pointing to a random discoverable <code>DoorLocker</code> device
         */
        public DoorLockerProxy anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>DoorLocker</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link DoorLockerComposite} object composed of all matching <code>DoorLocker</code> devices
         */
        public DoorLockerComposite whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>DoorLocker</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return a {@link DoorLockerComposite} object composed of all matching <code>DoorLocker</code> devices
         */
        public DoorLockerComposite whereLocation(java.lang.String location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
        
        /**
         * Returns a composite of all accessible <code>DoorLocker</code> devices whose attribute <code>user</code> matches a given value.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return a {@link DoorLockerComposite} object composed of all matching <code>DoorLocker</code> devices
         */
        public DoorLockerComposite whereUser(java.lang.String user) throws CompositeException {
            return instantiateComposite().andUser(user);
        }
    }
    
    /**
     * A composite of several <code>DoorLocker</code> devices
     * <p>
     * batteryLevel
     * 
     * <pre>
     * device DoorLocker extends PhysicalDevice {
     * 	action Lock;
     * 	action UnLock;
     * 	source locked as Boolean;
     * 	source state as State; 
     * 	source batstate as State; 
     * }
     * </pre>
     */
    protected final static class DoorLockerComposite extends fr.inria.diagen.core.service.composite.Composite<DoorLockerProxy> {
        private DoorLockerComposite(Service serviceParent) {
            super(serviceParent, "/Device/Device/PhysicalDevice/DoorLocker/");
        }
        
        @Override
        protected DoorLockerProxy instantiateProxy(RemoteServiceInfo rsi) {
            return new DoorLockerProxy(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link DoorLockerComposite}, filtered using the attribute <code>id</code>.
         */
        public DoorLockerComposite andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return this {@link DoorLockerComposite}, filtered using the attribute <code>location</code>.
         */
        public DoorLockerComposite andLocation(java.lang.String location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>user</code>.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return this {@link DoorLockerComposite}, filtered using the attribute <code>user</code>.
         */
        public DoorLockerComposite andUser(java.lang.String user) throws CompositeException {
            filterByAttribute("user", user);
            return this;
        }
        
        /**
         * Subscribes to the <code>locked</code> source. After a call to this method, the context will be notified when a
         * <code>DoorLocker</code> device of this composite publishes a value on its <code>locked</code> source.
         */
        public void subscribeLocked() {
            subscribeValue("locked");
        }
        
        /**
         * Unsubscribes from the <code>locked</code> source. After a call to this method, the context will no more be notified
         * when a <code>DoorLocker</code> device of this composite publishes a value on its <code>locked</code> source.
         */
        public void unsubscribeLocked() {
            unsubscribeValue("locked");
        }
        
        /**
         * Subscribes to the <code>batstate</code> source. After a call to this method, the context will be notified when a
         * <code>DoorLocker</code> device of this composite publishes a value on its <code>batstate</code> source.
         */
        public void subscribeBatstate() {
            subscribeValue("batstate");
        }
        
        /**
         * Unsubscribes from the <code>batstate</code> source. After a call to this method, the context will no more be notified
         * when a <code>DoorLocker</code> device of this composite publishes a value on its <code>batstate</code> source.
         */
        public void unsubscribeBatstate() {
            unsubscribeValue("batstate");
        }
        
        /**
         * Subscribes to the <code>state</code> source. After a call to this method, the context will be notified when a
         * <code>DoorLocker</code> device of this composite publishes a value on its <code>state</code> source.
         */
        public void subscribeState() {
            subscribeValue("state");
        }
        
        /**
         * Unsubscribes from the <code>state</code> source. After a call to this method, the context will no more be notified
         * when a <code>DoorLocker</code> device of this composite publishes a value on its <code>state</code> source.
         */
        public void unsubscribeState() {
            unsubscribeValue("state");
        }
    }
    
    /**
     * A proxy to one <code>DoorLocker</code> device
     * <p>
     * batteryLevel
     * 
     * <pre>
     * device DoorLocker extends PhysicalDevice {
     * 	action Lock;
     * 	action UnLock;
     * 	source locked as Boolean;
     * 	source state as State; 
     * 	source batstate as State; 
     * }
     * </pre>
     */
    protected final static class DoorLockerProxy extends Proxy {
        private DoorLockerProxy(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Subscribes to the <code>batstate</code> source. After a call to this method, the context will be notified when the
         * <code>DoorLocker</code> device of this proxy publishes a value on its <code>batstate</code> source.
         */
        public void subscribeBatstate() {
            subscribeValue("batstate");
        }
        
        /**
         * Unsubscribes from the <code>batstate</code> source. After a call to this method, the context will no more be notified
         * when the <code>DoorLocker</code> device of this proxy publishes a value on its <code>batstate</code> source.
         */
        public void unsubscribeBatstate() {
            unsubscribeValue("batstate");
        }
        
        /**
         * Subscribes to the <code>locked</code> source. After a call to this method, the context will be notified when the
         * <code>DoorLocker</code> device of this proxy publishes a value on its <code>locked</code> source.
         */
        public void subscribeLocked() {
            subscribeValue("locked");
        }
        
        /**
         * Unsubscribes from the <code>locked</code> source. After a call to this method, the context will no more be notified
         * when the <code>DoorLocker</code> device of this proxy publishes a value on its <code>locked</code> source.
         */
        public void unsubscribeLocked() {
            unsubscribeValue("locked");
        }
        
        /**
         * Subscribes to the <code>state</code> source. After a call to this method, the context will be notified when the
         * <code>DoorLocker</code> device of this proxy publishes a value on its <code>state</code> source.
         */
        public void subscribeState() {
            subscribeValue("state");
        }
        
        /**
         * Unsubscribes from the <code>state</code> source. After a call to this method, the context will no more be notified
         * when the <code>DoorLocker</code> device of this proxy publishes a value on its <code>state</code> source.
         */
        public void unsubscribeState() {
            unsubscribeValue("state");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
    // End of discover part for DoorLocker devices
}
