package fr.inria.phoenix.diasuite.framework.controller.mailnotifier;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.context.battery.BatteryValue;

/**
 * Send email messages about non-urgent events
 * 
 * <pre>
 * controller MailNotifier {
 * 	when provided Battery do SendMessage on Mailer;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractMailNotifier extends Service {
    
    public AbstractMailNotifier(ServiceConfiguration serviceConfiguration) {
        super("/Controller/MailNotifier/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        subscribeValue("battery", "/Context/Battery/"); // subscribe to Battery context
        postInitialize();
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("battery") && source.isCompatible("/Context/Battery/")) {
            BatteryValue batteryValue = new BatteryValue((java.lang.String) value);
            
            onBattery(batteryValue, new DiscoverForBattery());
        }
    }
    // End of methods from the Service class
    
    // Interaction contract implementation
    /**
     * This method is called when the <code>Battery</code> context publishes a value.
     * 
     * <pre>
     * when provided Battery do SendMessage on Mailer;
     * </pre>
     * 
     * @param battery the value of the <code>Battery</code> context.
     * @param discover a discover object to get context values and action methods
     */
    protected abstract void onBattery(BatteryValue battery, DiscoverForBattery discover);
    
    // End of interaction contract implementation
    
    // Discover object for Battery
    /**
     * An object to discover devices and contexts for the following interaction contract:
     * 
     * <code>
     * when provided Battery do SendMessage on Mailer;
     * </code>
     */
    protected final class DiscoverForBattery {
        private final MailerDiscovererForBattery mailerDiscoverer = new MailerDiscovererForBattery(AbstractMailNotifier.this);
        
        /**
         * @return a {@link MailerDiscovererForBattery} object to discover <code>Mailer</code> devices
         */
        public MailerDiscovererForBattery mailers() {
            return mailerDiscoverer;
        }
    }
    
    /**
     * Discover object that will exposes the <code>Mailer</code> devices to execute action on for the
     * <code>when provided Battery</code> interaction contract.
     * 
     * <pre>
     * device Mailer extends CommunicationService {
     * }
     * </pre>
     */
    protected final static class MailerDiscovererForBattery {
        private Service serviceParent;
        
        private MailerDiscovererForBattery(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private MailerCompositeForBattery instantiateComposite() {
            return new MailerCompositeForBattery(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>Mailer</code> devices
         * 
         * @return a {@link MailerCompositeForBattery} object composed of all discoverable <code>Mailer</code>
         */
        public MailerCompositeForBattery all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>Mailer</code> devices
         * 
         * @return a {@link MailerProxyForBattery} object pointing to a random discoverable <code>Mailer</code> device
         */
        public MailerProxyForBattery anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>Mailer</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link MailerCompositeForBattery} object composed of all matching <code>Mailer</code> devices
         */
        public MailerCompositeForBattery whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
    }
    
    /**
     * A composite of several <code>Mailer</code> devices to execute action on for the
     * <code>when provided Battery</code> interaction contract.
     * 
     * <pre>
     * device Mailer extends CommunicationService {
     * }
     * </pre>
     */
    protected final static class MailerCompositeForBattery extends fr.inria.diagen.core.service.composite.Composite<MailerProxyForBattery> {
        private MailerCompositeForBattery(Service serviceParent) {
            super(serviceParent, "/Device/Device/Service/CommunicationService/Mailer/");
        }
        
        @Override
        protected MailerProxyForBattery instantiateProxy(RemoteServiceInfo rsi) {
            return new MailerProxyForBattery(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link MailerCompositeForBattery}, filtered using the attribute <code>id</code>.
         */
        public MailerCompositeForBattery andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Executes the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> action's method on all devices of this composite.
         * 
         * @param to the <code>to</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param title the <code>title</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param content the <code>content</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param attachments the <code>attachments</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         */
        public void sendMessage(fr.inria.phoenix.diasuite.framework.datatype.contact.Contact to,
                java.lang.String title,
                java.lang.String content,
                java.util.List<fr.inria.phoenix.diasuite.framework.datatype.file.File> attachments) throws InvocationException {
            launchDiscovering();
            for (MailerProxyForBattery proxy : proxies) {
                proxy.sendMessage(to, title, content, attachments);
            }
        }
        
        /**
         * Executes the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> action's method on all devices of this composite.
         * 
         * @param to the <code>to</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param title the <code>title</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param content the <code>content</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param image the <code>image</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         */
        public void sendMessageWithImage(fr.inria.phoenix.diasuite.framework.datatype.contact.Contact to,
                java.lang.String title,
                java.lang.String content,
                java.lang.String image) throws InvocationException {
            launchDiscovering();
            for (MailerProxyForBattery proxy : proxies) {
                proxy.sendMessageWithImage(to, title, content, image);
            }
        }
    }
    
    /**
     * A proxy to one <code>Mailer</code> device to execute action on for the
     * <code>when provided Battery</code> interaction contract.
     * 
     * <pre>
     * device Mailer extends CommunicationService {
     * }
     * </pre>
     */
    protected final static class MailerProxyForBattery extends Proxy {
        private MailerProxyForBattery(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Executes the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> action's method on this device.
         * 
         * @param to the <code>to</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param title the <code>title</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param content the <code>content</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param attachments the <code>attachments</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         */
        public void sendMessage(fr.inria.phoenix.diasuite.framework.datatype.contact.Contact to,
                java.lang.String title,
                java.lang.String content,
                java.util.List<fr.inria.phoenix.diasuite.framework.datatype.file.File> attachments) throws InvocationException {
            callOrder("sendMessage", to, title, content, attachments);
        }
        
        /**
         * Executes the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> action's method on this device.
         * 
         * @param to the <code>to</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param title the <code>title</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param content the <code>content</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param image the <code>image</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         */
        public void sendMessageWithImage(fr.inria.phoenix.diasuite.framework.datatype.contact.Contact to,
                java.lang.String title,
                java.lang.String content,
                java.lang.String image) throws InvocationException {
            callOrder("sendMessageWithImage", to, title, content, image);
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
    }
    // End of discover object for Battery
}
