package fr.inria.phoenix.diasuite.framework.controller.smsnotifier;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.context.alert.AlertValue;

/**
 * Send SMS messages about urgent events
 * 
 * <pre>
 * controller SmsNotifier {
 * 	when provided Alert do SendMessage on SMSSender;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractSmsNotifier extends Service {
    
    public AbstractSmsNotifier(ServiceConfiguration serviceConfiguration) {
        super("/Controller/SmsNotifier/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        subscribeValue("alert", "/Context/Alert/"); // subscribe to Alert context
        postInitialize();
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("alert") && source.isCompatible("/Context/Alert/")) {
            AlertValue alertValue = new AlertValue((java.lang.String) value);
            
            onAlert(alertValue, new DiscoverForAlert());
        }
    }
    // End of methods from the Service class
    
    // Interaction contract implementation
    /**
     * This method is called when the <code>Alert</code> context publishes a value.
     * 
     * <pre>
     * when provided Alert do SendMessage on SMSSender;
     * </pre>
     * 
     * @param alert the value of the <code>Alert</code> context.
     * @param discover a discover object to get context values and action methods
     */
    protected abstract void onAlert(AlertValue alert, DiscoverForAlert discover);
    
    // End of interaction contract implementation
    
    // Discover object for Alert
    /**
     * An object to discover devices and contexts for the following interaction contract:
     * 
     * <code>
     * when provided Alert do SendMessage on SMSSender;
     * </code>
     */
    protected final class DiscoverForAlert {
        private final SMSSenderDiscovererForAlert sMSSenderDiscoverer = new SMSSenderDiscovererForAlert(AbstractSmsNotifier.this);
        
        /**
         * @return a {@link SMSSenderDiscovererForAlert} object to discover <code>SMSSender</code> devices
         */
        public SMSSenderDiscovererForAlert sMSSenders() {
            return sMSSenderDiscoverer;
        }
    }
    
    /**
     * Discover object that will exposes the <code>SMSSender</code> devices to execute action on for the
     * <code>when provided Alert</code> interaction contract.
     * 
     * <pre>
     * device SMSSender extends CommunicationService {
     * }
     * </pre>
     */
    protected final static class SMSSenderDiscovererForAlert {
        private Service serviceParent;
        
        private SMSSenderDiscovererForAlert(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private SMSSenderCompositeForAlert instantiateComposite() {
            return new SMSSenderCompositeForAlert(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>SMSSender</code> devices
         * 
         * @return a {@link SMSSenderCompositeForAlert} object composed of all discoverable <code>SMSSender</code>
         */
        public SMSSenderCompositeForAlert all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>SMSSender</code> devices
         * 
         * @return a {@link SMSSenderProxyForAlert} object pointing to a random discoverable <code>SMSSender</code> device
         */
        public SMSSenderProxyForAlert anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>SMSSender</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link SMSSenderCompositeForAlert} object composed of all matching <code>SMSSender</code> devices
         */
        public SMSSenderCompositeForAlert whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
    }
    
    /**
     * A composite of several <code>SMSSender</code> devices to execute action on for the
     * <code>when provided Alert</code> interaction contract.
     * 
     * <pre>
     * device SMSSender extends CommunicationService {
     * }
     * </pre>
     */
    protected final static class SMSSenderCompositeForAlert extends fr.inria.diagen.core.service.composite.Composite<SMSSenderProxyForAlert> {
        private SMSSenderCompositeForAlert(Service serviceParent) {
            super(serviceParent, "/Device/Device/Service/CommunicationService/SMSSender/");
        }
        
        @Override
        protected SMSSenderProxyForAlert instantiateProxy(RemoteServiceInfo rsi) {
            return new SMSSenderProxyForAlert(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link SMSSenderCompositeForAlert}, filtered using the attribute <code>id</code>.
         */
        public SMSSenderCompositeForAlert andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Executes the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> action's method on all devices of this composite.
         * 
         * @param to the <code>to</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param title the <code>title</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param content the <code>content</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param attachments the <code>attachments</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         */
        public void sendMessage(fr.inria.phoenix.diasuite.framework.datatype.contact.Contact to,
                java.lang.String title,
                java.lang.String content,
                java.util.List<fr.inria.phoenix.diasuite.framework.datatype.file.File> attachments) throws InvocationException {
            launchDiscovering();
            for (SMSSenderProxyForAlert proxy : proxies) {
                proxy.sendMessage(to, title, content, attachments);
            }
        }
        
        /**
         * Executes the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> action's method on all devices of this composite.
         * 
         * @param to the <code>to</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param title the <code>title</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param content the <code>content</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param image the <code>image</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         */
        public void sendMessageWithImage(fr.inria.phoenix.diasuite.framework.datatype.contact.Contact to,
                java.lang.String title,
                java.lang.String content,
                java.lang.String image) throws InvocationException {
            launchDiscovering();
            for (SMSSenderProxyForAlert proxy : proxies) {
                proxy.sendMessageWithImage(to, title, content, image);
            }
        }
    }
    
    /**
     * A proxy to one <code>SMSSender</code> device to execute action on for the
     * <code>when provided Alert</code> interaction contract.
     * 
     * <pre>
     * device SMSSender extends CommunicationService {
     * }
     * </pre>
     */
    protected final static class SMSSenderProxyForAlert extends Proxy {
        private SMSSenderProxyForAlert(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Executes the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> action's method on this device.
         * 
         * @param to the <code>to</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param title the <code>title</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param content the <code>content</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param attachments the <code>attachments</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         */
        public void sendMessage(fr.inria.phoenix.diasuite.framework.datatype.contact.Contact to,
                java.lang.String title,
                java.lang.String content,
                java.util.List<fr.inria.phoenix.diasuite.framework.datatype.file.File> attachments) throws InvocationException {
            callOrder("sendMessage", to, title, content, attachments);
        }
        
        /**
         * Executes the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> action's method on this device.
         * 
         * @param to the <code>to</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param title the <code>title</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param content the <code>content</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param image the <code>image</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         */
        public void sendMessageWithImage(fr.inria.phoenix.diasuite.framework.datatype.contact.Contact to,
                java.lang.String title,
                java.lang.String content,
                java.lang.String image) throws InvocationException {
            callOrder("sendMessageWithImage", to, title, content, image);
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
    }
    // End of discover object for Alert
}
