package fr.inria.phoenix.diasuite.framework.misc;

import fr.inria.diagen.core.deploy.AbstractDeploy;

import fr.inria.phoenix.diasuite.framework.context.alert.AbstractAlert;
import fr.inria.phoenix.diasuite.framework.context.battery.AbstractBattery;
import fr.inria.phoenix.diasuite.framework.context.format.AbstractFormat;

import fr.inria.phoenix.diasuite.framework.controller.log.AbstractLog;
import fr.inria.phoenix.diasuite.framework.controller.mailnotifier.AbstractMailNotifier;
import fr.inria.phoenix.diasuite.framework.controller.smsnotifier.AbstractSmsNotifier;

/**
 * This class should be implemented to bind the implementation of the various components
 */
public abstract class AppComponentBinder extends AbstractDeploy {

    // Context instances
    private AbstractAlert alertInstance = null;
    private AbstractBattery batteryInstance = null;
    private AbstractFormat formatInstance = null;

    // Controller instances
    private AbstractLog logInstance = null;
    private AbstractMailNotifier mailNotifierInstance = null;
    private AbstractSmsNotifier smsNotifierInstance = null;
    
    @Override
    public void deployAll() {
        // Initialization of contexts
        if (alertInstance == null)
            alertInstance = getInstance(getAlertClass());
        if (batteryInstance == null)
            batteryInstance = getInstance(getBatteryClass());
        if (formatInstance == null)
            formatInstance = getInstance(getFormatClass());
        // Intialization of controllers
        if (logInstance == null)
            logInstance = getInstance(getLogClass());
        if (mailNotifierInstance == null)
            mailNotifierInstance = getInstance(getMailNotifierClass());
        if (smsNotifierInstance == null)
            smsNotifierInstance = getInstance(getSmsNotifierClass());
        // Deploying contexts
        deploy(alertInstance);
        deploy(batteryInstance);
        deploy(formatInstance);
        // Deploying controllers
        deploy(logInstance);
        deploy(mailNotifierInstance);
        deploy(smsNotifierInstance);
    }
    
    @Override
    public void undeployAll() {
        // Undeploying contexts
        undeploy(alertInstance);
        undeploy(batteryInstance);
        undeploy(formatInstance);
        // Undeploying controllers
        undeploy(logInstance);
        undeploy(mailNotifierInstance);
        undeploy(smsNotifierInstance);
    }
    
    // Abstract binding methods for contexts
    /**
     * Overrides this method to provide the implementation class of the <code>Alert</code> context
     * <p>
     * Detect urgent events such as a forced door, a stuck door, or a blackout
     * 
     * <pre>
     * context Alert as String {
     * 	when provided state from DoorLocker maybe publish;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractAlert} that implements the <code>Alert</code> context
     */
    public abstract Class<? extends AbstractAlert> getAlertClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>Battery</code> context
     * <p>
     * Detect non-urgent events concerning a battery going low
     * 
     * <pre>
     * context Battery as String {
     * 	when provided batstate from DoorLocker maybe publish;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractBattery} that implements the <code>Battery</code> context
     */
    public abstract Class<? extends AbstractBattery> getBatteryClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>Format</code> context
     * <p>
     * * DoorLocks contexts
     * Format various events for logging purpose
     * 
     * <pre>
     * context Format as Event {
     * 	when provided locked from DoorLocker always publish;
     * 	when provided batstate from DoorLocker always publish;
     * 	when provided state from DoorLocker always publish;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractFormat} that implements the <code>Format</code> context
     */
    public abstract Class<? extends AbstractFormat> getFormatClass();
    
    // End of abstract binding methods for contexts
    
    // Abstract binding methods for controllers
    /**
     * Overrides this method to provide the implementation class of the <code>Log</code> controller
     * <p>
     * * DoorLocks controllers
     * Log events
     * 
     * <pre>
     * controller Log {
     * 	when provided Format do Log on Logger;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractLog} that implements the <code>Log</code> controller
     */
    public abstract Class<? extends AbstractLog> getLogClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>MailNotifier</code> controller
     * <p>
     * Send email messages about non-urgent events
     * 
     * <pre>
     * controller MailNotifier {
     * 	when provided Battery do SendMessage on Mailer;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractMailNotifier} that implements the <code>MailNotifier</code> controller
     */
    public abstract Class<? extends AbstractMailNotifier> getMailNotifierClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>SmsNotifier</code> controller
     * <p>
     * Send SMS messages about urgent events
     * 
     * <pre>
     * controller SmsNotifier {
     * 	when provided Alert do SendMessage on SMSSender;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractSmsNotifier} that implements the <code>SmsNotifier</code> controller
     */
    public abstract Class<? extends AbstractSmsNotifier> getSmsNotifierClass();
    
    // End of abstract binding methods for controllers
}
